/**
 * Title:        The Grid Market Directory (GMD) Toolkit
 * Developer:    The Gridbus Project
 *               The Grid Computing and Distributed Systems (GRIDS) Laboratory
 *               The University of Melbourne, Australia
 * URL:          http://www.gridbus.org
 * @version      Alpha Version,
 * Description:  The GMD Toolkit allows the creation of portal for management of
 *               Grid Market services. It also provide webservices based access
 *               to GMD from external applications.
 * @author       Jia Yu (program developer) and Rajkumar Buyya (project director)
 *
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */
package gridbus.gmd.service;

import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.Properties;


public class DatabaseHandler {
	private Connection connection;	
	private Statement statement;	
	
	
	public DatabaseHandler() throws ClassNotFoundException,SQLException{
	 			
		String DBDriver=null;
		String url=null;
		String username=null;
		String password=null;
		
		Properties props = new Properties();
		String pFile="GMDDB.prop";
		try {
                props.load(new FileInputStream(pFile));
            } catch(IOException e) {
                //throw new GMDQueryException("Failed to load in the property file: " + pFile);
                System.err.println("Failed to load property file");
            }
        Enumeration enum = props.propertyNames();
        while (enum.hasMoreElements()) {
             String key = (String)enum.nextElement();
             if (System.getProperty(key) == null) {
                // command line properties take precedence
             	System.setProperty(key, props.getProperty(key));
             }
        }
        password=System.getProperty("gridbus.gmd.DBPassword");
        username=System.getProperty("gridbus.gmd.DBUserName");
        url=System.getProperty("gridbus.gmd.DBURL");
        DBDriver=System.getProperty("gridbus.gmd.DBDriver");
        
    		try{		
			Class.forName(DBDriver);
			connection=DriverManager.getConnection(url,username,password);
		    statement=connection.createStatement();
		}catch(ClassNotFoundException e){
			System.err.println("Failed to load JDBC/ODBC driver.");
			e.printStackTrace();
		}catch(SQLException e){
			System.err.println("Unable to connect");
			e.printStackTrace();
		}
	}
	
	/**
	 * Method getNoOfServices.
	 * @param providerName
	 * @return int
	 * @throws DBQueryException
	 */
	public int getNoOfServices(String pn) throws DBQueryException{
		String query="SELECT COUNT(*) FROM GSPS WHERE Provider_Name='"+pn+"'";
		ResultSet r=null;
		try{
			r=executeQuery(query);
		}catch(DBQueryException e){
			throw e;
		}
		int c=0;
		try {
			c=r.getInt(1);
		} catch (SQLException e) {
			
		}
		return c;	
				
	}
	
	/**
	 * Method registerProvider.
	 * @param pn
	 * @param ln
	 * @param pw
	 * @return String
	 */
	public String registerProvider(String pn,String ln,String pw){
		String result;
		try{
			String insertion="INSERT INTO GSPR (Provider_Name,Login_Name,Password) VALUES ('"+
						pn+"','"+ln+"','"+pw+"')";
				   executeUpdate(insertion);
		}catch(DBQueryException e){
				result=new String("error when adding the new provider");
		}
		result=new String("OK");				 
		return result;

	}
	
	/**
	 * Method getProviderList.
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getProviderList()throws DBQueryException{
		String query="SELECT Service_Name,Service_Type, Provider_Name, Hardware_Price, Software_Price, Contact, Description FROM GSPS";
		ResultSet r=null;
		r=executeQuery(query);
		
		return r;
	}
	
	/**
	 * Method getServiceTypeList.
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getServiceTypeList() throws DBQueryException{
		String query="SELECT Service_Type FROM STS ORDER BY Service_Type ASC";
		ResultSet r=null;
		
		r=executeQuery(query);
		
		return r;
	}
	
	/**
	 * Method addOneService.
	 * @param sName
	 * @param sType
	 * @param pName
	 * @param hPrice
	 * @param sPrice
	 * @param contact
	 * @param description
	 * @return String
	 */
	public String addOneService(String sName,String sType,String pName,String hPrice,String sPrice,String contact,String description){
		float hp=Float.parseFloat(hPrice);
		float sp=Float.parseFloat(sPrice);
		
		String result=null;
		
		try{
			String insertion="INSERT INTO GSPS (Service_Name,Service_Type,Provider_Name,Hardware_Price,Software_Price,Contact,Description) VALUES ('"+
						sName+"','"+sType+"','"+pName+"',"+hp+","+sp+",'"+contact+"','"+description+"')";
				   executeUpdate(insertion);
		}catch(DBQueryException e){
				result=new String("error when adding the new service");
		}
		result=new String("OK");				 
		return result;
				
	}
	/**
	 * Method updateService.
	 * @param sName
	 * @param sType
	 * @param pName
	 * @param hPrice
	 * @param sPrice
	 * @param contact
	 * @param description
	 * @return String
	 */
	public String updateService(String sName,String sType,String pName,String hPrice,String sPrice,String contact,String description){
		float hp=Float.parseFloat(hPrice);
		float sp=Float.parseFloat(sPrice);
		
		String result=null;
		try{
			String updateStatement="UPDATE GSPS SET Service_Type='"+sType+"', Provider_Name='"+pName+"',Hardware_Price="+hPrice+
							", Software_Price="+sPrice+", Contact='"+contact+"', Description='"+description+"' WHERE Service_Name='"+sName+"'";
			executeUpdate(updateStatement);
		}catch(DBQueryException e){
			return new String("error when update the service");
		}
		return new String("OK");	
	}
	
	/**
	 * Method getServiceDetail.
	 * @param serviceName
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getServiceDetail(String serviceName) throws DBQueryException{
		String query="SELECT Service_Name,Service_Type, Provider_Name, Hardware_Price, Software_Price, Contact, Description FROM GSPS WHERE Service_Name='"+serviceName+"'";
		ResultSet r=null;
		
		r=executeQuery(query);
		
		return r;
	}
	
	/**
	 * Method getServiceList.
	 * @param providerName
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getServiceList(String providerName) throws DBQueryException{
		String query="SELECT Service_Name FROM GSPS WHERE Provider_Name='"+providerName+"'";
		ResultSet r=null;
		
		r=executeQuery(query);
		
		return r;
	}
	
	/**
	 * Method getAllProviders.
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getAllProviders() throws DBQueryException{
		String query="SELECT Provider_Name FROM GSPR";
		ResultSet r=null;
		
		r=executeQuery(query);
		
		return r;
	}
	
	/**
	 * Method removeOneService.
	 * @param serviceName
	 * @return String
	 */
	public String removeOneService(String serviceName){
		String deleteStatement="DELETE FROM GSPS WHERE Service_Name='"+serviceName+"'";
		try{
			executeUpdate(deleteStatement);
		}catch(DBQueryException e){
			return new String("ERROR");
		}
		return new String("OK");	
	}


	private ResultSet executeQuery(String query) throws DBQueryException{
		boolean moreRecord=false;
		ResultSet resultSet;
		
		try{
				resultSet=statement.executeQuery(query);
				moreRecord=resultSet.next();
		}catch(SQLException e){
			throw new DBQueryException("DB Query error"+e);  
		}
		if(!moreRecord){
			throw new DBQueryException("No matching record");
		}else{
				return resultSet;
		}
	}
	
	private void executeUpdate(String updateStatement) throws DBQueryException{
		try {
  			statement.executeUpdate(updateStatement);
  			} catch (SQLException se) {
			 	throw new DBQueryException(se.getMessage());
  			}
	}
	
	/**
	 * Method getProviderName.
	 * @param string
	 * @param string1
	 * @return String
	 */
	public String getProviderName(String ln,String pw) {
		String query="SELECT Provider_Name FROM GSPR WHERE Login_Name='"+ln+"' and Password='"+pw+"'";
		ResultSet r=null;
		try{
			r=executeQuery(query);
		}catch(DBQueryException e){
			return new String("ERROR");
		}
		String pn=null;
		try{
			pn=r.getString(1);
		}catch(SQLException e){
			return new String("ERROR");
		}
		return pn;
	}

	/**
	 * Method removeProvider.
	 * @param string
	 * @return String
	 */
	public String removeProvider(String providerName) {
		
		String deleteStatement="DELETE FROM GSPR WHERE Provider_Name='"+providerName+"'";
		
		try{
			executeUpdate(deleteStatement);
		}catch(DBQueryException e){
			return new String("ERROR");
		}
		deleteStatement="DELETE FROM GSPS WHERE Provider_Name='"+providerName+"'";
		try{
			executeUpdate(deleteStatement);
		}catch(DBQueryException e){
			return new String("ERROR");
		}
		
		return new String("OK");	
	}

	/**
	 * Method checkProvider.
	 * @param providerName
	 * @return boolean
	 */
	public boolean checkProvider(String providerName){
		String query="SELECT Provider_Name FROM GSPR WHERE Provider_Name='"+providerName+"'";
		ResultSet r=null;
		try{
			r=executeQuery(query);
		}catch(DBQueryException e){
			if(e.getMessage().equals("No matching record")) return true;
			else	return false;
		}
		return false;
	}
	
	/**
	 * Method checkLoginName.
	 * @param loginName
	 * @return boolean
	 */
	public boolean checkLoginName(String loginName){
		String query="SELECT Provider_Name FROM GSPR WHERE Login_Name='"+loginName+"'";
		ResultSet r=null;
		try{
			r=executeQuery(query);
		}catch(DBQueryException e){
			if(e.getMessage().equals("No matching record")) return true;
			else	return false;
		}
		return false;
	}
	/**
	 * Method getProviderServices.
	 * @param providerName
	 * @return ResultSet
	 */
	public ResultSet getProviderServices(String providerName) throws DBQueryException{
		ResultSet r=null;
		String query="SELECT Service_Name, Service_Type, Hardware_Price, Software_Price, Contact, Description FROM GSPS WHERE Provider_Name='"+providerName+"'";
		
		r=executeQuery(query);
		
		return r;
	}
	/**
	 * Method getServicetypeServices.
	 * @param serviceType
	 * @return ResultSet
	 */
	public ResultSet getServicetypeServices(String serviceType) throws DBQueryException {
		ResultSet r=null;
		String query="SELECT Service_Name, Provider_Name, Hardware_Price, Software_Price, Contact, Description FROM GSPS WHERE Service_Type='"+serviceType+"'";
		
		r=executeQuery(query);
		
		return r;
	}

}
