/**
 * Title:        The Grid Market Directory (GMD) Toolkit
 * Developer:    The Gridbus Project
 *               The Grid Computing and Distributed Systems (GRIDS) Laboratory
 *               The University of Melbourne, Australia
 * URL:          http://www.gridbus.org
 * @version      Alpha Version,
 * Description:  The GMD Toolkit allows the creation of portal for management of
 *               Grid Market services. It also provide webservices based access
 *               to GMD from external applications.
 * @author       Jia Yu (program developer) and Rajkumar Buyya (project director)
 *
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */
package gridbus.gmd.service;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Vector;


public class WebIF {
	
	private DatabaseHandler db=null;
	
	
	public WebIF(){
		try{
			db=new DatabaseHandler();
		}catch(ClassNotFoundException e){
		}catch(SQLException e){
		}
	}
	
	
	public int countProviderServices(String providerName) throws DBQueryException{
		return db.getNoOfServices(providerName.trim());
	
	}
	
	/**
	 * Method registerProvider.
	 * @param providerName
	 * @param loginName
	 * @param password
	 * @return String
	 */
	public String registerProvider(String providerName,String loginName,String password){
		return db.registerProvider(providerName.trim(),loginName.trim(),password.trim());
	}
	
		
	/**
	 * Method getProviderName.
	 * @param loginName
	 * @param password
	 * @return String
	 */
	public String getProviderName(String loginName,String password){
		return db.getProviderName(loginName.trim(),password.trim());
	}
	
	/**
	 * Method unregisterProvider.
	 * @param providerName
	 * @return String
	 */
	public String unregisterProvider(String providerName){
		return db.removeProvider(providerName.trim());
	}

	/**
	 * Method checkProvider.
	 * @param providerName
	 * @return boolean
	 */
	public boolean checkProvider(String providerName){
		return db.checkProvider(providerName.trim());
		
	}	
	
	/**
	 * Method checkLoginName.
	 * @param loginName
	 * @return boolean
	 */
	public boolean checkLoginName(String loginName){
		return db.checkLoginName(loginName.trim());
	}
	
	/**
	 * Method getProviderList.
	 * @return TableSet
	 */
	public TableSet getProviderList()throws DBQueryException{
			return getTable(db.getProviderList());
	}
	
	
	/**
	 * Method getServiceList.
	 * @param providerName
	 * @return ListIterator
	 * @throws DBQueryException
	 */
	public ListIterator getServiceList(String providerName) throws DBQueryException{
			
			LinkedList sl=new LinkedList();
			ResultSet r=null;
			
			try{
				r=db.getServiceList(providerName);
					
				do{
					sl.add(r.getString(1));
			
				}while(r.next());

			}catch(SQLException e){
				throw new DBQueryException(e.getMessage());
			}
			return sl.listIterator(); 
	}
	
	/**
	 * Method getServiceTypeList.
	 * @return ListIterator
	 */
	public ListIterator getServiceTypeList() throws DBQueryException{
			LinkedList stl=new LinkedList();
			ResultSet r=null;
			
			r=db.getServiceTypeList();
			try{
				
				do{
					stl.add(r.getString(1));
				}while(r.next());
			} catch (SQLException e) {
				throw new DBQueryException(e.getMessage());
			}
			return stl.listIterator();
	}
	
	/**
	 * Method getAllProviders.
	 * @return ListIterator
	 * @throws DBQueryException
	 */
	public ListIterator getAllProviders() throws DBQueryException{
			LinkedList pl=new LinkedList();
			ResultSet r=null;
			
			try{
				r=db.getAllProviders();
				do{
					pl.add(r.getString(1));
				}while(r.next());
			}catch(SQLException e){
				throw new DBQueryException(e.getMessage());
			}
			return pl.listIterator();
	}
	
	/**
	 * Method getProviderServiceTable.
	 * @param providerName
	 * @return TableSet
	 * @throws DBQueryException
	 */
	public TableSet getProviderServiceTable(String providerName) throws DBQueryException{
			return getTable(db.getProviderServices(providerName));				
	}
	
	/**
	 * Method getServiceTypeTable.
	 * @param serviceType
	 * @return TableSet
	 * @throws DBQueryException
	 */
	public TableSet getServiceTypeTable(String serviceType) throws DBQueryException{
			return getTable(db.getServicetypeServices(serviceType));
	
	}
	
	/**
	 * Method addOneService.
	 * @param serviceName:String
	 * @param serviceType:String
	 * @param providerName:String
	 * @param hardwarePrice:String
	 * @param softwarePrice:String
	 * @param contact:String
	 * @param description:String
	 * @return String (OK,if the process is success) 
	 */
	public String addOneService(String sName,String sType,String pName, String hPrice, String sPrice,String contact,String description){
		    return db.addOneService(sName.trim(),sType.trim(),pName.trim(),hPrice.trim(),sPrice.trim(),contact.trim(),description.trim());
	}
	
	/**
	 * Method updateService.
	 * @param sName
	 * @param sType
	 * @param pName
	 * @param hPrice
	 * @param sPrice
	 * @param contact
	 * @param description
	 * @return String
	 */
	public String updateService(String sName,String sType,String pName, String hPrice, String sPrice,String contact,String description){
			return db.updateService(sName.trim(),sType.trim(),pName.trim(),hPrice.trim(),sPrice.trim(),contact.trim(),description.trim());
	}
	
	/**
	 * Method removeService.
	 * @param sName
	 * @return String
	 */
	public String removeService(String sName){
			return db.removeOneService(sName.trim());
	}
	
	
	
	/**
	 * Method getServiceDetail.
	 * @param serviceName
	 * @return ServiceDetail
	 * @throws DBQueryException
	 */
	public ServiceDetail getServiceDetail(String serviceName) throws DBQueryException{
		ResultSet r=null;
		try{
			r=db.getServiceDetail(serviceName);
		}catch(DBQueryException e){
			throw e;
		}
		ResultSetMetaData rmd;
		try {
			rmd = r.getMetaData();
		} catch (SQLException e) {
			throw new DBQueryException(e.getMessage());
		}
		
		String[] type={"Service_Name","Service_Type","Provider_Name","Hardware_Price","Software_Price","Contact","Description"};
		ServiceDetail sd=new ServiceDetail();
		
		try {
			for(int i=1;i<=rmd.getColumnCount();i++){
			    int t=-1;
				for(int c=0;c<rmd.getColumnCount();c++){
					if(rmd.getColumnName(i).equals(type[c])){
						t=c;
						break;
					}
				}
				switch(t){
					case 0:
						sd.setServiceName(r.getString(i));
						break;
					case 1:
						sd.setServiceType(r.getString(i));
						break;
					case 2:
						sd.setProviderName(r.getString(i));
						break;
					case 3:
						sd.setHardwarePrice(Float.toString(r.getFloat(i)));
						break;
					case 4:
						sd.setSoftwarePrice(Float.toString(r.getFloat(i)));
						break;
					case 5:
					 	sd.setAddress(r.getString(i));
					 	break;
					case 6:
						sd.setDescription(r.getString(i));
						break;
					default:
					    throw new DBQueryException("Unknown column type");
				}
			}
		} catch (SQLException e) {
			throw new DBQueryException(e.getMessage());
		}
		return sd;
	}
	
	
	private TableSet getTable(ResultSet r){
		Vector columnHeads=new Vector();
		Vector rows=new Vector();
		
		try{
			//get column heads
			ResultSetMetaData rmd=r.getMetaData();
			
			for(int i=1;i<=rmd.getColumnCount();++i)
				columnHeads.addElement(rmd.getColumnName(i));
				//get row data
			do{
				rows.addElement(getNextRow(r,rmd));
		    }while(r.next());
		}catch(SQLException e){
			e.printStackTrace();
		}
		return new TableSet(columnHeads,rows);
	}
		  
	
	private Vector getNextRow(ResultSet rs, ResultSetMetaData rsmd) throws SQLException {
		Vector currentRow=new Vector();
		for(int i=1;i<=rsmd.getColumnCount();++i)
			switch(rsmd.getColumnType(i)){
				case Types.VARCHAR:
					currentRow.addElement(rs.getString(i));
				  break;
				case Types.INTEGER:
					currentRow.addElement((new Long(rs.getLong(i))).toString());
				  break;
				case Types.FLOAT:
				    currentRow.addElement((new Float(rs.getFloat(i))).toString());
				  break;
				default:
					System.out.println("Type was:"+rsmd.getColumnTypeName(i));
		}
		return currentRow;
	}
}
