import gridbus.gmd.soap.gmdqueryclient.GMDQuery;
import gridbus.gmd.soap.gmdqueryclient.GMDQueryException;
import gridbus.gmd.soap.gmdqueryclient.GMDService;
import gridbus.gmd.soap.gmdqueryclient.Price;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.ListIterator;
import java.util.Properties;

import com.sun.jini.madison.util.*;

/**
 * Title:        The Grid Market Directory (GMD) Toolkit
 * Developer:    The Gridbus Project
 *               The Grid Computing and Distributed Systems (GRIDS) Laboratory
 *               The University of Melbourne, Australia
 * URL:          http://www.gridbus.org
 * @version      Alpha Version,
 * Description:  The GMD Toolkit allows the creation of portal for management of
 *               Grid Market services. It also provide webservices based access
 *               to GMD from external applications.
 * @author       Jia Yu (program developer) and Rajkumar Buyya (project director)
 *
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */

public class gridgmd {

	public static void main(String[] args) {
		
		String gmdHost=null;
		String port=null;
		String soapDir=null;
		String soapServiceName=null;
		
		// start to parse command line arguments
		ArrayList flags = new ArrayList();
        flags.add(new String[]{"-gmdHostName", "-h"});
        flags.add(new String[]{"-serviceType","-st"});
		flags.add(new String[]{"-serviceName","-sn"});
		flags.add(new String[]{"-serviceHostName","-sh"});
		flags.add(new String[]{"-serviceProvider","-sp"});
		flags.add(new String[]{"-allService","-a"});
		
	    Properties props = new Properties();
		String pFile="GMDQuery.prop";
		try {
                props.load(new FileInputStream(pFile));
            } catch(IOException e) {
        	System.err.println(e.getMessage());
        }
        Enumeration enum = props.propertyNames();
        while (enum.hasMoreElements()) {
             String key = (String)enum.nextElement();
             if (System.getProperty(key) == null) {
                // command line properties take precedence
             	System.setProperty(key, props.getProperty(key));
             }
        }
        gmdHost=System.getProperty("gridbus.gmd.GMDHostAddress");
        port=System.getProperty("gridbus.gmd.GMDPort");
        soapDir=System.getProperty("gridbus.gmd.SOAPDirectory");
        soapServiceName=System.getProperty("gridbus.gmd.SOAPQueryService");
   
		
		FlagSpec[] flagSpecArray = new FlagSpec[flags.size()];
        for (int i = 0; i < flags.size(); i++) {
            flagSpecArray[i] = new FlagSpec((String[])flags.get(i), false,true, new StringArgType());
        }
        CommandLine commandLine = new CommandLine(flagSpecArray, args);
	    if(commandLine.isArgPresent("-gmdHostName")){
	    	gmdHost=commandLine.getFlagArg("-gmdHostName");
	    }
	    
	  
			
		if (commandLine.isArgPresent("-serviceType")) {
 			String sType = commandLine.getFlagArg("-serviceType");
			
			GMDQuery g=null;
			ListIterator l=null;	
	    	GMDService s=null;	
			
 			try{   
                                long st=System.currentTimeMillis();
 				g=new GMDQuery(gmdHost,Integer.parseInt(port),soapDir,soapServiceName);
				l=g.getServiceListByType(sType.trim());
                                long ft=System.currentTimeMillis();
                                System.out.println("Done: "+(ft-st));
			}catch(GMDQueryException e){
				System.err.println(e.getMessage());
				System.exit(1);
			}
		
			//displayTableTitle();
		
			do{
				s=(GMDService)l.next();
				System.out.println(s.toString());
			}while(l.hasNext());
 		}else if(commandLine.isArgPresent("-serviceName")){
 			String sName=commandLine.getFlagArg("-serviceName");

 			GMDQuery g=null;
                 
                        long time=0;
                        for(int i=0;i<100;i++){
 			try{
				g=new GMDQuery(gmdHost,Integer.parseInt(port),soapDir,soapServiceName);
                                Price p=null;
                                long st=System.currentTimeMillis();
				p=g.getPrice(sName.trim());
                                long ft=System.currentTimeMillis();
                                if(i!=0){
                                      time=(ft-st)+time;
				}
                              //  System.out.println("Done: "+ (ft-st));
		//		System.out.println("Service Name: "+sName);
		    	//System.out.println(p.toString());
			}catch(GMDQueryException e){
				System.err.println(e.getMessage());
				System.exit(2);
			}
                         
                        }//for	
                        System.out.println("Average time="+(time/99));
 			
 		}else if(commandLine.isArgPresent("-serviceHostName")){
 			String hName=commandLine.getFlagArg("-serviceHostName");
 			GMDQuery g=null;
 			GMDService s=null;
 			try{
 				g=new GMDQuery(gmdHost,Integer.parseInt(port),soapDir,soapServiceName);
				s=g.getServiceByHost(hName.trim());
				System.out.println("Grid Node Name: "+hName);
			//	displayTableTitle();
		    	System.out.println(s.toString());
			}catch(GMDQueryException e){
				System.err.println(e.getMessage());
				System.exit(3);
			}
 		}else if(commandLine.isArgPresent("-allService")){
		  	GMDQuery g=null;
		  	ListIterator l=null;	
	        	GMDService s=null;	
                        try{
   	 			long st = System.currentTimeMillis();
				g=new GMDQuery(gmdHost,Integer.parseInt(port),soapDir,soapServiceName);
 			  	long ft = System.currentTimeMillis();
                        	System.out.println("Done: " + (ft - st));
				l=g.getServiceList();
			}catch(GMDQueryException e){
				System.err.println(e.getMessage());
				System.exit(4);
			}
				
			do{
				s=(GMDService)l.next();
				//System.out.println(s.toString());
			
			}while(l.hasNext());
 		}else if(commandLine.isArgPresent("-serviceProvider")){
 			GMDQuery g=null;
 			GMDService s=null;
 			ListIterator l=null;	
 			System.out.println("Please enter provider name:");

 			BufferedReader br=new BufferedReader(new InputStreamReader(System.in));
            String pName=null;
			try {
				pName=br.readLine().trim();
			} catch (IOException e) {
				System.err.println(e.getMessage());
			}
			System.out.println(pName);
                        for(int i=0;i<10;i++){
 			try{
                                
				long st=System.currentTimeMillis();
                                System.out.println("st"+st);
 				g=new GMDQuery(gmdHost,Integer.parseInt(port),soapDir,soapServiceName);
				l=g.getServiceListByProvider(pName.trim());
                                long ft=System.currentTimeMillis();
                                System.out.println("ft"+ft);
                                System.out.println("Done: "+(ft-st));
				//displayTableTitle();
			/*	do{
				   s=(GMDService)l.next();
				   System.out.println(s.toString());
				   System.out.println("");
			    }while(l.hasNext());*/
		    }catch(GMDQueryException e){
				System.err.println(e.getMessage());
				System.exit(3);
			}
                    }//for
		}else{ 			
 			displayHelp();
		}
		
	}

	/**
	 * Method displayTableTitle.
	 */
	private static void displayTableTitle(){
			StringBuffer tName=new StringBuffer(10);
			tName.append("Name");
			StringBuffer tType=new StringBuffer(25);
			tType.append("Type");
			StringBuffer tProvider=new StringBuffer(20);
			tProvider.append("Provider");
			StringBuffer tAddress=new StringBuffer(25);
			tAddress.append("Host");
			StringBuffer tHP=new StringBuffer("$Hardware");
			StringBuffer tSP=new StringBuffer("$Software");
			System.out.println(tName+"\t\t"+tType+"\t\t\t"+tProvider+"\t\t"+tAddress+"\t\t"+tHP+"\t"+tSP+"\t"+"Path");
			System.out.println("");

	}
	
	private static void displayHelp(){
		System.out.println("Usage:gridgmd [-h]||[-sh] [-sn] [-st] [-sp]");
		System.out.println("Options");
		System.out.println("-a:\t\t\t"+"Display all services in the Grid Market Directory");
		System.out.println("-h  GMD Host Name:\t"+"The host address of the Grid Market Directory" );
		System.out.println("-sh Service Host Name:\t"+"Query a service by its service host name");
		System.out.println("-sn Service Name:\t"+"Query a service by its service name");
		System.out.println("-st Service Type Name:\t"+"Query services by the service type");
		System.out.println("-sp:"+"\t\t\t"+"Query services by the service provider name");		
	}
}
