/**
 * Title:        The Grid Market Directory (GMD) Toolkit
 * Developer:    The Gridbus Project
 *               The Grid Computing and Distributed Systems (GRIDS) Laboratory
 *               The University of Melbourne, Australia
 * URL:          http://www.gridbus.org
 * @version      Alpha Version,
 * Description:  The GMD Toolkit allows the creation of portal for management of
 *               Grid Market services. It also provide webservices based access
 *               to GMD from external applications.
 * @author       Jia Yu (program developer) and Rajkumar Buyya (project director)
 *
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */
package gridbus.gmd.soap.service;

import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Enumeration;
import java.util.Properties;


public class DatabaseHandler {
	private Connection connection;	
	private Statement statement;	
	
	
	public DatabaseHandler() throws ClassNotFoundException,SQLException{
	 	String DBDriver=null;
		String url=null;
		String username=null;
		String password=null;
		
		Properties props = new Properties();
		String pFile="GMDDB.prop";
		try {
                props.load(new FileInputStream(pFile));
            } catch(IOException e) {
                //throw new GMDQueryException("Failed to load in the property file: " + pFile);
                System.err.println("Failed to load property file");
            }
        Enumeration enum = props.propertyNames();
        while (enum.hasMoreElements()) {
             String key = (String)enum.nextElement();
             if (System.getProperty(key) == null) {
                // command line properties take precedence
             	System.setProperty(key, props.getProperty(key));
             }
        }
        password=System.getProperty("gridbus.gmd.DBPassword");
        username=System.getProperty("gridbus.gmd.DBUserName");
        url=System.getProperty("gridbus.gmd.DBURL");
        DBDriver=System.getProperty("gridbus.gmd.DBDriver");
        
    	
		try{		
			Class.forName(DBDriver);
			connection=DriverManager.getConnection(url,username,password);
		    statement=connection.createStatement();
		}catch(ClassNotFoundException e){
			System.err.println("Failed to load JDBC/ODBC driver.");
			e.printStackTrace();
		}catch(SQLException e){
			System.err.println("Unable to connect");
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Method getServiceContact.
	 * @param serviceType
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getServiceContact(String serviceType) throws DBQueryException{
		String query="SELECT Service_Name, Contact FROM GSPS WHERE Service_Type='"+serviceType+"'";
		ResultSet r=executeQuery(query);

		return r;
	}

	/**
	 * Method getServices.
	 * @param serviceType
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getServices(String serviceType) throws DBQueryException{
		String query="SELECT Service_Name, Provider_Name, Hardware_Price, Software_Price, Contact, Description FROM GSPS WHERE Service_Type='"+serviceType+"'";

		ResultSet r=executeQuery(query);
		return r;
	}
	/**
	 * Method getProviderServices.
	 * @param providerName
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getProviderServices(String providerName) throws DBQueryException{
		String query="SELECT Service_Name,Service_Type,Hardware_Price, Software_Price, Contact, Description FROM GSPS WHERE Provider_Name='"+providerName+"'";
		
		ResultSet r=executeQuery(query);
		return r;
	}
	
	/**
	 * Method getServices.
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getServices() throws DBQueryException{
		String query="SELECT Service_Name, Service_Type, Provider_Name, Hardware_Price, Software_Price, Contact, Description FROM GSPS";

		ResultSet r=executeQuery(query);
		return r;
	}
	

	/**
	 * Method getServiceByHost.
	 * @param hostName
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getServiceByHost(String hostName) throws DBQueryException{
		String query="SELECT Service_Name, Service_Type, Provider_Name, Hardware_Price, Software_Price, Description FROM GSPS WHERE Contact='"+hostName+"'";

		ResultSet r=executeQuery(query);
		return r;
	}
	
	/**
	 * Method getPrice.
	 * @param serviceName
	 * @return ResultSet
	 * @throws DBQueryException
	 */
	public ResultSet getPrice(String serviceName) throws DBQueryException{
		String query="SELECT Hardware_Price,Software_Price FROM GSPS WHERE Service_Name='"+serviceName+"'";
		
		ResultSet r=executeQuery(query);
				
		return r;
	}
	
	/**
	 * Method closeDB.
	 */
	public void closeDB(){
		try {
				if ( connection!= null) connection.close();
			}
		catch (SQLException e) {}
	}
	
	private ResultSet executeQuery(String query) throws DBQueryException{
		boolean moreRecord=false;
		ResultSet resultSet;
		
		try{
				resultSet=statement.executeQuery(query);
				moreRecord=resultSet.next();
		}catch(SQLException e){
			throw new DBQueryException("DB Query error"+e);  
		}
		if(!moreRecord){
			throw new DBQueryException("No matching record");
		}else{
				return resultSet;
		}
	}
	
	
}
