/**
 * Title:        The Grid Market Directory (GMD) Toolkit
 * Developer:    The Gridbus Project
 *               The Grid Computing and Distributed Systems (GRIDS) Laboratory
 *               The University of Melbourne, Australia
 * URL:          http://www.gridbus.org
 * @version      Alpha Version,
 * Description:  The GMD Toolkit allows the creation of portal for management of
 *               Grid Market services. It also provide webservices based access
 *               to GMD from external applications.
 * @author       Jia Yu (program developer) and Rajkumar Buyya (project director)
 *
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */
package gridbus.gmd.soap.service;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;
import org.jdom.JDOMException;

import java.io.StringReader;
import java.io.StringWriter;
import java.sql.ResultSet;
import java.sql.SQLException;

public class GMDService {


		public GMDService(){
			
			System.out.println("GMD start");
		}
		
		/**
		 * Method queryServiceContact.
		 * @param inputReq
		 * @return String
		 */
		public String queryServiceContact(String inputReq){
			
			//create the root of the return xml 
			Document ServiceContactDetails=null;
			XMLOutputter outputter=null;
			Element ServiceContactDetailsRoot=new Element("SERVICE-DETAILS");
			
			//open a connection to the database
			DatabaseHandler dh=null;
			
			try{
				dh=new DatabaseHandler();
			}catch(ClassNotFoundException e){
				ServiceContactDetailsRoot.setAttribute("STATUS","ERROR");
				ServiceContactDetailsRoot.addContent(new Element("REASON").setText("error with Database handler"));
				ServiceContactDetails=new Document(ServiceContactDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServiceContactDetails);
			}catch(SQLException e){
				ServiceContactDetailsRoot.setAttribute("STATUS","ERROR");
				ServiceContactDetailsRoot.addContent(new Element("REASON").setText("error with Database connection"));
				ServiceContactDetails=new Document(ServiceContactDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServiceContactDetails);
			}
			
			//request parsing	
			SAXBuilder builder=new SAXBuilder();
			Document ServiceContactReq=null;		
			try{
				ServiceContactReq=builder.build(new StringReader(inputReq));
			}catch(JDOMException e){
				ServiceContactDetailsRoot.setAttribute("STATUS","ERROR");
				ServiceContactDetailsRoot.addContent(new Element("REASON").setText("request parsing error"));
				ServiceContactDetails=new Document(ServiceContactDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServiceContactDetails);
			}
			
			Element serviceContactReqRoot=ServiceContactReq.getRootElement();
			String sType=serviceContactReqRoot.getChild("SERVICE_TYPE").getText();
	
			//database query
			ResultSet r=null;
			try{
				r=dh.getServiceContact(sType);
			}catch(DBQueryException e){
				ServiceContactDetailsRoot.setAttribute("STATUS","ERROR");
				ServiceContactDetailsRoot.addContent(new Element("REASON").setText(e.getMessage()));
				ServiceContactDetails=new Document(ServiceContactDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServiceContactDetails);
			}
			
			//construct the return xml	
			try{
				do{
					Element provider=new Element("SERVICE");
					Element name=new Element("NAME");
					name.setText(r.getString(1));
					provider.addContent(name);
					Element host=new Element("ADDRESS");
					host.setText(r.getString(2));
					provider.addContent(host);
					ServiceContactDetailsRoot.addContent(provider);
				}while(r.next());
			}catch(SQLException e){
				ServiceContactDetailsRoot.setAttribute("STATUS", "ERROR");
				ServiceContactDetails=new Document(ServiceContactDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
            	return outputter.outputString(ServiceContactDetails);	
			}
			ServiceContactDetailsRoot.setAttribute("STATUS", "OK");
			ServiceContactDetails=new Document(ServiceContactDetailsRoot);
			outputter=new XMLOutputter(" ",true);
			dh.closeDB();
            return outputter.outputString(ServiceContactDetails);	
		}
		
		/**
		 * Method queryService.
		 * @return String
		 */
		public String queryService(){
			//create the root of the return xml
			Document ServicesDetails=null;
			XMLOutputter outputter=null;
			Element ServicesDetailsRoot=new Element("SERVICES-DETAILS");
			
			//open a connection to the database
			DatabaseHandler dh=null;
			
			try{
				dh=new DatabaseHandler();
			}catch(ClassNotFoundException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText("error with Database handler"));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}catch(SQLException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText("error with Database connection"));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}
			
			
			//database query
			ResultSet r=null;
			try{
				r=dh.getServices();
			}catch(DBQueryException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText(e.getMessage()));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}
			
			//construct the return xml	
			try{
				do{
					Element service=new Element("SERVICE");
					
					Element name=new Element("NAME");
					name.setText(r.getString(1));
					service.addContent(name);
					Element type=new Element("TYPE");
					type.setText(r.getString(2));
					service.addContent(type);
					Element provider=new Element("PROVIDER");
					provider.setText(r.getString(3));
					service.addContent(provider);
					
					Element price=new Element("PRICE");
					
					Element hp=new Element("HARDWARE");
					hp.setText(Float.toString(r.getFloat(4)));
					price.addContent(hp);
					
					Element sp=new Element("SOFTWARE");
					sp.setText(Float.toString(r.getFloat(5)));
					price.addContent(sp);
					
					service.addContent(price);
					
					Element host=new Element("ADDRESS");
					host.setText(r.getString(6));
					service.addContent(host);
					Element des=new Element("DESCRIPTION");
					des.setText(r.getString(7));
					service.addContent(des);
					
					ServicesDetailsRoot.addContent(service);
					
				}while(r.next());
				
			}catch(SQLException e){
				ServicesDetailsRoot.setAttribute("STATUS", "ERROR");
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
            	return outputter.outputString(ServicesDetails);	
			}
			ServicesDetailsRoot.setAttribute("STATUS", "OK");
			ServicesDetails=new Document(ServicesDetailsRoot);
			outputter=new XMLOutputter(" ",true);
			dh.closeDB();
            return outputter.outputString(ServicesDetails);	
		}
		
		/**
		 * Method queryServiceByType.
		 * @param inputReq
		 * @return String
		 */
		public String queryServiceByType(String inputReq){
			
			//create the root of the return xml 
			Document ServicesDetails=null;
			XMLOutputter outputter=null;
			Element ServicesDetailsRoot=new Element("SERVICES-DETAILS");
			
			//open a connection to the database
			DatabaseHandler dh=null;
			
			try{
				dh=new DatabaseHandler();
			}catch(ClassNotFoundException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText("error with Database handler"));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}catch(SQLException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText("error with Database connection"));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}
			
			//request parsing	
			SAXBuilder builder=new SAXBuilder();
			Document ServicesReq=null;		
			try{
				ServicesReq=builder.build(new StringReader(inputReq));
			}catch(JDOMException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText("request parsing error"));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}
			
			Element serviceReqRoot=ServicesReq.getRootElement();
			String sType=serviceReqRoot.getChild("SERVICE_TYPE").getText();
		
			ServicesDetailsRoot.setAttribute("TYPE",sType);
			
			//database query
			ResultSet r=null;
			try{
				r=dh.getServices(sType);
			}catch(DBQueryException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText(e.getMessage()));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}
			
			//construct the return xml	
			try{
				do{
					Element service=new Element("SERVICE");
					
					Element name=new Element("NAME");
					name.setText(r.getString(1));
					service.addContent(name);
					Element provider=new Element("PROVIDER");
					provider.setText(r.getString(2));
					service.addContent(provider);
					
					//price element with two children
					Element price=new Element("PRICE");
					
					Element hp=new Element("HARDWARE");
					hp.setText(Float.toString(r.getFloat(3)));
					price.addContent(hp);
					
					Element sp=new Element("SOFTWARE");
					sp.setText(Float.toString(r.getFloat(4)));
					price.addContent(sp);
					   
					service.addContent(price);
					
					Element host=new Element("ADDRESS");
					host.setText(r.getString(5));
					service.addContent(host);
					Element des=new Element("DESCRIPTION");
					des.setText(r.getString(6));
					service.addContent(des);
					
					ServicesDetailsRoot.addContent(service);
					
				}while(r.next());
				
			}catch(SQLException e){
				ServicesDetailsRoot.setAttribute("STATUS", "ERROR");
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
            	return outputter.outputString(ServicesDetails);	
			}
			ServicesDetailsRoot.setAttribute("STATUS", "OK");
			ServicesDetails=new Document(ServicesDetailsRoot);
			outputter=new XMLOutputter(" ",true);
			dh.closeDB();
            return outputter.outputString(ServicesDetails);	
		}
		
		/**
		 * Method queryServiceByProvider.
		 * @param inputReq
		 * @return String
		 */
		public String queryServiceByProvider(String inputReq){
				//create the root of the return xml 
			Document ServicesDetails=null;
			XMLOutputter outputter=null;
			Element ServicesDetailsRoot=new Element("SERVICES-DETAILS");
			
			//open a connection to the database
			DatabaseHandler dh=null;
			
			try{
				dh=new DatabaseHandler();
			}catch(ClassNotFoundException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText("error with Database handler"));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}catch(SQLException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText("error with Database connection"));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}
			
			//request parsing	
			SAXBuilder builder=new SAXBuilder();
			Document ServicesReq=null;		
			try{
				ServicesReq=builder.build(new StringReader(inputReq));
			}catch(JDOMException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText("request parsing error"));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}
			
			Element serviceReqRoot=ServicesReq.getRootElement();
			String pName=serviceReqRoot.getChild("PROVIDER_NAME").getText();
		
			ServicesDetailsRoot.setAttribute("PROVIDER",pName);
			
			//database query
			ResultSet r=null;
			try{
				r=dh.getProviderServices(pName);
			}catch(DBQueryException e){
				ServicesDetailsRoot.setAttribute("STATUS","ERROR");
				ServicesDetailsRoot.addContent(new Element("REASON").setText(e.getMessage()));
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServicesDetails);
			}
			
			//construct the return xml	
			try{
				do{
					Element service=new Element("SERVICE");
					
					Element name=new Element("NAME");
					name.setText(r.getString(1));
					service.addContent(name);
					Element provider=new Element("TYPE");
					provider.setText(r.getString(2));
					service.addContent(provider);
					
					//price element with two children
					Element price=new Element("PRICE");
					
					Element hp=new Element("HARDWARE");
					hp.setText(Float.toString(r.getFloat(3)));
					price.addContent(hp);
					
					Element sp=new Element("SOFTWARE");
					sp.setText(Float.toString(r.getFloat(4)));
					price.addContent(sp);
					   
					service.addContent(price);
					
					Element host=new Element("ADDRESS");
					host.setText(r.getString(5));
					service.addContent(host);
					Element des=new Element("DESCRIPTION");
					des.setText(r.getString(6));
					service.addContent(des);
					
					ServicesDetailsRoot.addContent(service);
					
				}while(r.next());
				
			}catch(SQLException e){
				ServicesDetailsRoot.setAttribute("STATUS", "ERROR");
				ServicesDetails=new Document(ServicesDetailsRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
            	return outputter.outputString(ServicesDetails);	
			}
			ServicesDetailsRoot.setAttribute("STATUS", "OK");
			ServicesDetails=new Document(ServicesDetailsRoot);
			outputter=new XMLOutputter(" ",true);
			dh.closeDB();
            return outputter.outputString(ServicesDetails);		
		
		}
		
		/**
		 * Method queryServiceByHost.
		 * @param inputReq
		 * @return String
		 */
		public String queryServiceByHost(String inputReq){
			//construct the root of the return xml
			Document ServiceDetail=null;
			XMLOutputter outputter=null;
			Element ServiceDetailRoot=new Element("SERVICE-DETAIL");
		

			//open a database connection
			DatabaseHandler dh=null;
			try{
				dh=new DatabaseHandler();
			}catch(ClassNotFoundException e){
				ServiceDetailRoot.setAttribute("STATUS","ERROR");
				ServiceDetailRoot.addContent(new Element("REASON").setText("error with Database handler"));
				ServiceDetail=new Document(ServiceDetailRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServiceDetail);				
			}catch(SQLException e){
				ServiceDetailRoot.setAttribute("STATUS","ERROR");
				ServiceDetailRoot.addContent(new Element("REASON").setText("error with Database connection"));
				ServiceDetail=new Document(ServiceDetailRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServiceDetail);				

			}

     		//request parsing
			SAXBuilder builder=new SAXBuilder();
			Document HostReq=null;
			
			try{
				HostReq=builder.build(new StringReader(inputReq));
			}catch(JDOMException e){
				ServiceDetailRoot.setAttribute("STATUS","ERROR");
				ServiceDetailRoot.addContent(new Element("REASON").setText("error with request parsing"));
				ServiceDetail=new Document(ServiceDetailRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServiceDetail);				
			}
			Element hostReqRoot=HostReq.getRootElement();
			String hName=hostReqRoot.getChild("HOST_NAME").getText();

			ServiceDetailRoot.setAttribute("HOST",hName);

			//Database query
			ResultSet r=null;
			try {
				r = dh.getServiceByHost(hName);
			} catch (DBQueryException e) {
				ServiceDetailRoot.setAttribute("STATUS","ERROR");
				ServiceDetailRoot.addContent(new Element("REASON").setText(e.getMessage()));
				ServiceDetail=new Document(ServiceDetailRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServiceDetail);				
			}
			
			//construct the return xml				
			try{
				ServiceDetailRoot.addContent(new Element("NAME").setText(r.getString(1)));
				ServiceDetailRoot.addContent(new Element("TYPE").setText(r.getString(2)));
				ServiceDetailRoot.addContent(new Element("PROVIDER").setText(r.getString(3)));
				ServiceDetailRoot.addContent(new Element("HPRICE").setText(Float.toString(r.getFloat(4))));
				ServiceDetailRoot.addContent(new Element("SPRICE").setText(Float.toString(r.getFloat(5))));
				ServiceDetailRoot.addContent(new Element("DESCRIPTION").setText(r.getString(6)));
			}catch(SQLException e){
				ServiceDetailRoot.setAttribute("STATUS","ERROR");
				ServiceDetailRoot.addContent(new Element("REASON").setText("Data Retriving Error"));
				ServiceDetail=new Document(ServiceDetailRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(ServiceDetail);
			}
			ServiceDetailRoot.setAttribute("STATUS", "OK");
			ServiceDetail=new Document(ServiceDetailRoot);
			outputter=new XMLOutputter(" ",true);
			dh.closeDB();
			return outputter.outputString(ServiceDetail);
		}

		
		
	
		/**
		 * Method queryPrice.
		 * @param inputReq
		 * @return String
		 */
		public String queryPrice(String inputReq){
			//construct the root of the return xml
			Document PriceDetail=null;
			XMLOutputter outputter=null;
			Element PriceDetailRoot=new Element("PRICE-DETAIL");
		

			//open a database connection
			DatabaseHandler dh=null;
			try{
				dh=new DatabaseHandler();
			}catch(ClassNotFoundException e){
				PriceDetailRoot.setAttribute("STATUS","ERROR");
				PriceDetailRoot.addContent(new Element("REASON").setText("error with Database handler"));
				PriceDetail=new Document(PriceDetailRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(PriceDetail);				
			}catch(SQLException e){
				PriceDetailRoot.setAttribute("STATUS","ERROR");
				PriceDetailRoot.addContent(new Element("REASON").setText("error with Database connection"));
				PriceDetail=new Document(PriceDetailRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(PriceDetail);				

			}

     		//request parsing
			SAXBuilder builder=new SAXBuilder();
			Document PriceReq=null;
			
			try{
				PriceReq=builder.build(new StringReader(inputReq));
			}catch(JDOMException e){
				PriceDetailRoot.setAttribute("STATUS","ERROR");
				PriceDetailRoot.addContent(new Element("REASON").setText("error with request parsing"));
				PriceDetail=new Document(PriceDetailRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(PriceDetail);				
			}
			Element priceReqRoot=PriceReq.getRootElement();
			String sName=priceReqRoot.getChild("SERVICE_NAME").getText();

			PriceDetailRoot.setAttribute("SERVICE",sName);

			//Database query
			ResultSet r=null;
			try {
				r = dh.getPrice(sName);
			} catch (DBQueryException e) {
				PriceDetailRoot.setAttribute("STATUS","ERROR");
				PriceDetailRoot.addContent(new Element("REASON").setText(e.getMessage()));
				PriceDetail=new Document(PriceDetailRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(PriceDetail);				
			}
			
			//construct the return xml				
			try{
				PriceDetailRoot.addContent(new Element("HARDWARE").setText(Float.toString(r.getFloat(1))));
				PriceDetailRoot.addContent(new Element("SOFTWARE").setText(Float.toString(r.getFloat(2))));
			}catch(SQLException e){
				PriceDetailRoot.setAttribute("STATUS","ERROR");
				PriceDetailRoot.addContent(new Element("REASON").setText("Data Retriving Error"));
				PriceDetail=new Document(PriceDetailRoot);
				outputter=new XMLOutputter(" ",true);
				dh.closeDB();
				return outputter.outputString(PriceDetail);
			}
			PriceDetailRoot.setAttribute("STATUS", "OK");
			PriceDetail=new Document(PriceDetailRoot);
			outputter=new XMLOutputter(" ",true);
			dh.closeDB();
			return outputter.outputString(PriceDetail);
		}
		
	}
